@@ -61,14 +61,14 @@ import re
 from base64 import b64decode
 from wsgiref.headers import Headers
 
-from slimta.logging import log_exception
+from slimta import logging
 from slimta.http.wsgi import WsgiServer
 from slimta.envelope import Envelope
 from slimta.smtp.reply import Reply
 from slimta.queue import QueueError
 from slimta.relay import RelayError
 from slimta.util.ptrlookup import PtrLookup
-from . import Edge
+from . import EdgeServer
 
 __all__ = ['WsgiResponse', 'WsgiEdge', 'WsgiValidators']
 
@@ -111,11 +111,17 @@ def _build_http_response(smtp_reply):
         return WsgiResponse('500 Internal Server Error', headers)
 
 
-class WsgiEdge(Edge, WsgiServer):
+class WsgiEdge(EdgeServer, WsgiServer):
     """This class is intended to be instantiated and used as an app on top of a
     WSGI server engine such as :class:`gevent.pywsgi.WSGIServer`. It will only
     acccept ``POST`` requests that provide a ``message/rfc822`` payload.
 
+    .. note::
+
+        If ``listener`` is not provided, use
+        :meth:`~slimta.http.WsgiServer.build_server` to build and manage the
+        server manually.
+
     :param queue: |Queue| object used by :meth:`.handoff()` to ensure the
                   envelope is properly queued before acknowledged by the edge
                   service.
@@ -127,6 +133,13 @@ class WsgiEdge(Edge, WsgiServer):
     :param uri_pattern: If given, only URI paths that match the given pattern
                         will be allowed.
     :type uri_pattern: :py:class:`~re.RegexObject` or :py:obj:`str`
+    :param listener: Usually a ``(ip, port)`` tuple defining the interface
+                     and port upon which to listen for connections.
+    :param pool: If given, defines a specific :class:`gevent.pool.Pool` to
+                 use for new greenlets.
+    :param context: Enables SSL encryption on connected sockets using the
+                    information given in the context.
+    :type context: :py:class:`~ssl.SSLContext`
 
     """
 
@@ -149,13 +162,18 @@ class WsgiEdge(Edge, WsgiServer):
     ehlo_header = 'X-Ehlo'
 
     def __init__(self, queue, hostname=None, validator_class=None,
-                 uri_pattern=None):
-        super(WsgiEdge, self).__init__(queue, hostname)
+                 uri_pattern=None, listener=None, pool=None, context=None):
+        super(WsgiEdge, self).__init__(None, queue, hostname=hostname)
         self.validator_class = validator_class
         if isinstance(uri_pattern, str):
             self.uri_pattern = re.compile(uri_pattern)
         else:
             self.uri_pattern = uri_pattern
+        if listener:
+            ssl_args = {'ssl_context': context}
+            self.server = self.build_server(listener, pool, ssl_args)
+        else:
+            self.server = None
 
     def __call__(self, environ, start_response):
         ptr_lookup = PtrLookup(environ.get('REMOTE_ADDR', '0.0.0.0'))
@@ -169,7 +187,7 @@ class WsgiEdge(Edge, WsgiServer):
             start_response(res.status, res.headers)
             return res.data
         except Exception as exc:
-            log_exception(__name__)
+            logging.log_exception(__name__)
             msg = '{0!s}\n'.format(exc)
             content_length = str(len(msg))
             headers = [('Content-Length', content_length),
